#include <msp/core/maputils.h>
#include <msp/fs/stat.h>
#include <msp/fs/utils.h>
#include <msp/io/file.h>
#include <msp/io/print.h>
#include <msp/time/utils.h>
#include "builder.h"
#include "config.h"
#include "sourcepackage.h"

using namespace std;
using namespace Msp;

Config::Config(SourcePackage &p):
	package(p),
	changed(false)
{ }

const Config::Option &Config::add_option(const Feature &f)
{
	Option opt(f);
	InputOptions::const_iterator i = pending_options.find(opt.name);
	if(i!=pending_options.end())
		opt.value = i->second;

	return options.insert(OptionMap::value_type(opt.name, opt)).first->second;
}

bool Config::set_option(const string &opt, const string &val)
{
	bool result = false;

	OptionMap::iterator i = options.find(opt);
	if(i!=options.end())
	{
		if(i->second.value!=val)
		{
			result = true;
			changed = true;
			mtime = Time::now();
		}
		i->second.value = val;
	}

	return result;
}

bool Config::is_option(const string &name) const
{
	return options.count(name);
}

const Config::Option &Config::get_option(const string &name) const
{
	return get_item(options, name);
}

void Config::load()
{
	FS::Path fn = package.get_source_directory()/".config";
	FS::Stat stat = FS::stat(fn);
	if(stat)
	{
		package.get_builder().get_logger().log("files", format("Reading %s", fn));
		IO::BufferedFile in(fn.str());

		mtime = stat.get_modify_time();

		DataFile::Parser parser(in, fn.str());
		Loader loader(*this);
		loader.load(parser);
	}
}

void Config::save() const
{
	if(!changed)
		return;

	FS::Path fn = package.get_source_directory()/".config";

	package.get_builder().get_logger().log("files", format("Writing %s", fn));
	IO::BufferedFile out(fn.str(), IO::M_WRITE);

	for(OptionMap::const_iterator i=options.begin(); i!=options.end(); ++i)
		IO::print(out, "option \"%s\" \"%s\";\n", i->second.name, i->second.value);
}


Config::Option::Option(const Feature &f):
	Feature(f),
	value(default_value)
{
	name = "with_"+name;
}


Config::Loader::Loader(Config &c):
	DataFile::ObjectLoader<Config>(c)
{
	add("option", &Loader::option);
}

void Config::Loader::option(const string &n, const string &v)
{
	if(obj.options.count(n))
		obj.set_option(n, v);
	else
		obj.pending_options[n] = v;
}
