#ifndef GNUCOMPILER_H_
#define GNUCOMPILER_H_

#include "tool.h"

/**
Common base class for GNU compilers.  Turns SourceFiles into ObjectFiles.

Since invocation is mostly the same for all language frontends, most of the
logic is here and the individual tools only handle creating source files of
appropriate type.
*/
class GnuCompiler: public Tool
{
protected:
	unsigned version;

public:
	GnuCompiler(Builder &, const Architecture &, const std::string &);

	virtual Target *create_source(const Component &, const Msp::FS::Path &) const;
	virtual Target *create_source(const Msp::FS::Path &) const;
	virtual Target *create_target(const std::list<Target *> &, const std::string &);
	virtual std::string create_build_signature(const BuildInfo &) const;
protected:
	virtual void do_prepare();
	void prepare_syspath();
	void prepare_version();
	void prepare_version(const std::string &);
public:
	virtual Task *run(const Target &) const;
};

#endif
